@@ -1,3 +1,4 @@
+from pathlib import Path
 from typing import Any, List
 
 
@@ -17,3 +18,8 @@ def get_target_feature(schema: dict, target: str, type_: str):
         return x[0]
 
     return list(map(get_first, filter(mask, schema[target].items())))
+
+
+def make_dirs(*args):
+    for dir_ in args:
+        Path(dir_).mkdir(parents=True, exist_ok=True)
